import tensorflow as tf
import numpy as np


class SaliencyMap(object):
    """Implementaion of Saliency Map with Vanilla Gradient

        https://arxiv.org/pdf/1312.6034.pdf

        Example Usage:
        >>> attr_fn = SaliencyMap(sess, input_tensor, output_tensor, 0)
        >>> saliency_map = attr_fn(X) <--- equivalent to attr_fn.__call__(X)
    """
    def __init__(self, sess, input_tensor, output_tensor, target_class):
        """__Constructor__

        Arguments:
            sess {tf.Session} -- Tensorflow session
            input_tensor {tf.Tensor} -- Symbolic tensor of input node
            output_tensor {tf.Tensor} -- Symbolic tensor of pre-softmax output
            target_class {int} -- The class of interest to run explanation.
        """
        self.sess = sess
        self.input_tensor = input_tensor
        self.output_tensor = output_tensor
        self.target_class = target_class

        self._define_ops()

    def _define_ops(self):
        """Add whatever operations you feel necessary into the computational
        graph.
        """

        # >>> Your code starts here <<<

        # >>> Your code ends here <<<
        raise NotImplementedError

    def __call__(self, X, batch_size=16, multiply_with_input=True):
        """__call__ forward computation to generate the saliency map

        Arguments:
            X {np.ndarray} -- Input dataset

        Keyword Arguments:
            batch_size {int} -- Batch size (default: {16})
            multiply_with_input {bool} -- If True, return grad x input,
                else return grad (default: {True})

        Returns np.ndarray of the same shape as X.
        """
        # >>> Your code starts here <<<

        # >>> Your code ends here <<<
        raise NotImplementedError


class IntegratedGrad(object):
    """Implementaion of Integrated Gradient

        https://arxiv.org/pdf/1703.01365.pdf

        Example Usage:
        >>> attr_fn = IntegratedGrad(sess, input_tensor, output_tensor, 0)
        >>> integrated_grad = attr_fn(X, 'black')

    """
    def __init__(self, sess, input_tensor, output_tensor, target_class):
        """__Constructor__

        Arguments:
            sess {tf.Session} -- Tensorflow session
            input_tensor {tf.Tensor} -- Symbolic tensor of input node
            output_tensor {tf.Tensor} -- Symbolic tensor of pre-softmax output
            target_class {int} -- The class of interest to run explanation.
        """
        self.sess = sess
        self.input_tensor = input_tensor
        self.output_tensor = output_tensor
        self.target_class = target_class

        self._define_ops()

    def _define_ops(self):
        """Add whatever operations you feel necessary into the computational
        graph.
        """
        # >>> Your code starts here <<<

        # >>> Your code ends here <<<
        raise NotImplementedError

    def __call__(self,
                 X,
                 baseline='black',
                 batch_size=16,
                 num_steps=50,
                 multiply_with_input=True):
        """__call__ forward computation to generate the integrated gradient

        Arguments:
            X {np.array} -- Input dataset

        Keyword Arguments:
            baseline {str} -- The baseline input. One of 'black', 'white',
                or 'random' (default: {'black'})
            batch_size {int} -- Batch size (default: {16})
            num_steps {int} -- resolution of using sum to approximate the
                integral (default: {50})
            multiply_with_input {bool} -- If True, return grad x input,
                else return grad (default: {True})

        Returns np.ndarray of the same shape as X.
        """
        # >>> Your code starts here <<<

        # >>> Your code ends here <<<
        raise NotImplementedError
